(* PIX322 version 1 *)

Unit Pix322;

{$G+}

Interface

Type
   BufP=^BufT;
	BufT=Array[0..63999] Of Byte;

	TPalEntry=Record
   	R,G,B:Byte;
   End;
   TPalette=Array[0..255] Of TPalEntry;

Var
	PixPal:TPalette;

(* Initialization & Closing routines *)
Function P_Open(NbPages:Byte):Boolean;
Procedure P_Close;

(* Non-drawing functions *)
Procedure P_SelPage(NPage:Byte);
Procedure P_Dump;
Procedure P_PCopy(Src,Dst:Byte);
Procedure P_VSync;
Procedure P_CopyRect(X1,Y1,X2,Y2:Integer;From,Too:Byte);
Procedure P_CopyRectXY(X1,Y1,X2,Y2,X,Y:Integer;From,Too:Byte);

(* Drawing functions *)
Procedure P_Clear(Col:Byte);
Procedure P_Plot(X,Y:Integer;Col:Byte);
Procedure P_HLine(X1,X2,Y1:Integer;Col:Byte);
Procedure P_VLine(Y1,Y2,X1:Integer;Col:Byte);
Procedure P_Line(X1,Y1,X2,Y2:Integer;Col:Byte);
Procedure P_Box(X1,Y1,X2,Y2:Integer;Col:Byte);
Procedure P_FBox(X1,Y1,X2,Y2:Integer;OutCol,InCol:Byte);
Function P_GetPixel(X,Y:Integer):Byte;

(* Palette management *)
Procedure P_SetRGB(Col,R,G,B:Byte);
Procedure P_GetRGB(Col:Byte;Var R,G,B:Byte);
Procedure P_SetPalette(Pal:TPalette);
Procedure P_GetPalette(Var Pal:TPalette);
Procedure P_Black(Var Pal:TPalette);
Procedure P_FadeOut(Var Pal:TPalette;Chan:Byte);
Procedure P_FadeIn(Var Pal:TPalette;DstPal:TPalette;Chan:Byte);
Procedure P_Rotate(Var Pal:TPalette;From,Too:Byte;Reverse:Boolean);
Procedure P_SavePal(Pal:TPalette;Fil:String);
Procedure P_LoadPal(Fil:String;Var Pal:TPalette);
Procedure P_ResetPalette(Var Pal:TPalette);

Var
	NbPage:Byte;
   Pages:Array[0..10] Of BufP;
   PSeg,POfs:Word;
   TruePal:TPalette;

Implementation

Function P_Open(NbPages:Byte):Boolean;
Var
	C,D:Integer;
Begin
   If (NbPages>10) Or (NbPages<1) Then
   Begin
   	P_Open:=False;
      Exit;
   End;
	Asm
		mov AH,0
   	mov AL,13h
      int 10h
	End;
	NbPage:=NbPages;
   For C:=1 To NbPages Do
   Begin
   	If MaxAvail<64000 Then
      Begin
      	P_Open:=False;
         For D:=1 To C-1 Do
         	Dispose(Pages[D]);
         NbPage:=0;
         Exit;
      End;
   	New(Pages[C]);
   End;
   Pages[0]:=Ptr($A000,$0000);
   P_Open:=True;
   For C:=0 To NbPages Do
   Begin
      P_SelPage(C);
   	P_Clear(0);
   End;
	P_SelPage(1);
   P_GetPalette(PixPal);
   P_GetPalette(TruePal);
End;

Procedure P_Close;
Var
	C:Integer;
Begin
	Asm
		mov AH,0
   	mov AL,3h
      int 10h
	End;
	For C:=1 To NbPage Do
   	Dispose(Pages[C]);
End;

Procedure P_SelPage(NPage:Byte);
Begin
   If NPage>NbPage Then
   Begin
   	P_Close;
      WriteLn(#7,'FATAL ERROR: Trying to access unallocate dpage.');
      Halt(0);
   End;
	PSeg:=Seg(Pages[NPage]^);
   POfs:=Ofs(Pages[NPage]^);
End;

Procedure P_Dump;
Begin
	Move(Ptr(PSeg,POfs)^,Pages[0]^,64000);
End;

Procedure P_PCopy(Src,Dst:Byte);
Begin
   If Src>NbPage Then
   Begin
   	P_Close;
      WriteLn(#7,'FATAL ERROR: Trying to access unallocate dpage.');
      Halt(0);
   End;
   If Dst>NbPage Then
   Begin
   	P_Close;
      WriteLn(#7,'FATAL ERROR: Trying to access unallocate dpage.');
      Halt(0);
   End;
	Move(Pages[Src]^,Pages[Dst]^,64000);
End;

Procedure P_VSync;
Begin
	While Port[$3DA] And $08=$08 Do Asm Nop End;
   While Port[$3DA] And $08<>$08 Do Asm Nop End;
End;

Procedure P_CopyRect(X1,Y1,X2,Y2:Integer;From,Too:Byte);
Var
	Wd,Hg:Integer;
   Ic:Integer;
   SSg,Sof,DSg,DOf:Word;
Begin
   If From>NbPage Then
   Begin
   	P_Close;
      WriteLn(#7,'FATAL ERROR: Trying to access unallocate dpage.');
      Halt(0);
   End;
   If Too>NbPage Then
   Begin
   	P_Close;
      WriteLn(#7,'FATAL ERROR: Trying to access unallocate dpage.');
      Halt(0);
   End;
	Wd:=X2-X1+1;
   Hg:=Y2-Y1+1;
   Ic:=320-Wd;
   SSg:=Seg(Pages[From]^);
   SOf:=Ofs(Pages[From]^)+(Y1 shl 8)+(Y1 shl 6)+X1;
   DSg:=Seg(Pages[Too]^);
   DOf:=Ofs(Pages[Too]^)+(Y1 shl 8)+(Y1 shl 6)+X1;
   Asm
      push DS
		mov AX,SSg
      mov DS,SSg
      mov SI,SOf
      mov AX,DSg
      mov ES,AX
      mov DI,DOf
      mov DX,Hg
     @CR1:
      mov CX,Wd
      rep movsb
      add SI,Ic
      add DI,Ic
		dec DX
      jnz @CR1
      pop DS
   End;
End;

Procedure P_CopyRectXY(X1,Y1,X2,Y2,X,Y:Integer;From,Too:Byte);
Var
	Wd,Hg:Integer;
   Ic:Integer;
   SSg,Sof,DSg,DOf:Word;
Begin
   If From>NbPage Then
   Begin
   	P_Close;
      WriteLn(#7,'FATAL ERROR: Trying to access unallocate dpage.');
      Halt(0);
   End;
   If Too>NbPage Then
   Begin
   	P_Close;
      WriteLn(#7,'FATAL ERROR: Trying to access unallocate dpage.');
      Halt(0);
   End;
	Wd:=X2-X1+1;
   Hg:=Y2-Y1+1;
   Ic:=320-Wd;
   SSg:=Seg(Pages[From]^);
   SOf:=Ofs(Pages[From]^)+(Y1 shl 8)+(Y1 shl 6)+X1;
   DSg:=Seg(Pages[Too]^);
   DOf:=Ofs(Pages[Too]^)+(Y shl 8)+(Y shl 6)+X;
   Asm
      push DS
		mov AX,SSg
      mov DS,SSg
      mov SI,SOf
      mov AX,DSg
      mov ES,AX
      mov DI,DOf
      mov DX,Hg
     @CRXY1:
      mov CX,Wd
      rep movsb
      add SI,Ic
      add DI,Ic
		dec DX
      jnz @CRXY1
      pop DS
   End;
End;

Procedure P_Clear(Col:Byte);
Begin
	FillChar(Ptr(PSeg,POfs)^,64000,Col);
End;

Procedure P_Plot(X,Y:Integer;Col:Byte);
Begin
	Asm
   	mov AX,[PSeg]
      mov ES,AX
      mov DI,[Y]
      mov BX,DI
      shl DI,8
      shl BX,6
      add DI,BX
      add DI,[X]
      add DI,[POfs]
      mov AL,[Col]
      mov ES:[DI],AL
	End;
End;

Function P_GetPixel(X,Y:Integer):Byte;
Begin
	P_GetPIxel:=Byte(Ptr(PSeg,POfs+(Y shl 8)+(Y shl 6)+X)^);
End;

Procedure P_HLine(X1,X2,Y1:Integer;Col:Byte);
Begin
	Asm
   	mov AX,[PSeg]
      mov ES,AX
      mov DI,[Y1]
      mov BX,DI
      shl DI,8
      shl BX,6
      add DI,BX
      add DI,[X1]
      add DI,[POfs]
      mov AL,[Col]
      mov CX,X2
      sub CX,X1
      inc CX
   	rep stosb
	End;
End;

Procedure P_VLine(Y1,Y2,X1:Integer;Col:Byte);
Begin
	Asm
   	mov AX,[PSeg]
      mov ES,AX
      mov DI,[Y1]
      mov BX,DI
      shl DI,8
      shl BX,6
      add DI,BX
      add DI,[X1]
      add DI,[POfs]
      mov AL,[Col]
      mov CX,Y2
      sub CX,Y1
      inc CX
     @VL1:
      mov ES:[DI],AL
      add DI,320
     loop @VL1
	End;
End;

Procedure P_Line(X1,Y1,X2,Y2:Integer;Col:Byte);
Var
	T:Integer;
   Slop,RS,Dx,Dy:Integer;
   RY1,RY2:Integer;
Begin
	If X1=X2 Then
   Begin
      If Y2<Y1 Then
      Begin
      	T:=Y2;
         Y2:=Y1;
         Y1:=T;
      End;
   	P_VLine(Y1,Y2,X1,Col);
      Exit;
   End;
	If Y1=Y2 Then
   Begin
      If X2<X1 Then
      Begin
      	T:=X2;
         X2:=X1;
         X1:=T;
      End;
   	P_HLine(X1,X2,Y1,Col);
      Exit;
   End;
   If X2<X1 Then
   Begin
		T:=X2;
	   X2:=X1;
	   X1:=T;
		T:=Y2;
	   Y2:=Y1;
	   Y1:=T;
	End;
	Dy:=Y2-Y1;
	Dx:=X2-X1;
   If Abs(Dy)<=Abs(Dx) Then
   Begin
	   Slop:=Trunc((Dy*100)div Dx);
	   RS:=Abs(Slop);
		Asm
	   	mov AX,[PSeg]
	      mov ES,AX
	      mov DI,[Y1]
	      mov BX,DI
	      shl DI,8
	      shl BX,6
	      add DI,BX
	      add DI,[X1]
	      add DI,[POfs]
	      mov AL,[Col]
	     	mov CX,X2
	      sub CX,X1
	      inc CX
	      xor DX,DX
	     @LN1:
	      mov ES:[DI],AL
	      add DX,[RS]
	      cmp DX,99
	      jg @LN2
	      inc DI
	      loop @LN1
	      jmp @LN3
	     @LN2:
	      sub DX,100
	      cmp slop,0
	      jg @LN4
	      sub DI,320
	      jmp @LN5
	     @LN4:
	      add DI,320
	     @LN5:
	      inc DI
	      loop @LN1
	     @LN3:
		End;
	End
   Else
   Begin
	   Slop:=Trunc((Dx*100)div Dy);
	   RS:=Abs(Slop);
      If Y1>Y2 Then
      Begin
      	RY1:=Y2;
         RY2:=Y1;
      End
   	Else
      Begin
      	RY1:=Y1;
         RY2:=Y2;
      End;
		Asm
	   	mov AX,[PSeg]
	      mov ES,AX
	      mov DI,[Y1]
	      mov BX,DI
	      shl DI,8
	      shl BX,6
	      add DI,BX
	      add DI,[X1]
	      add DI,[POfs]
	      mov AL,[Col]
	     	mov CX,RY2
	      sub CX,RY1
	      inc CX
	      xor DX,DX
	     @LN1B:
	      mov ES:[DI],AL
         cmp slop,0
         jg @LN4B
         sub DI,320
         jmp @LN5B
        @LN4B:
        	add DI,320
        @LN5B:
	      add DX,[RS]
        	cmp DX,99
         jg @LN2B
	      loop @LN1B
	      jmp @LN3B
        @LN2B:
        	sub DX,100
         inc DI
         loop @LN1B
	     @LN3B:
		End;
   End;
End;

Procedure P_Box(X1,Y1,X2,Y2:Integer;Col:Byte);
Begin
	P_Line(X1,Y1,X2,Y1,Col);
	P_Line(X1,Y2,X2,Y2,Col);
	P_Line(X1,Y1,X1,Y2,Col);
	P_Line(X2,Y1,X2,Y2,Col);
End;

Procedure P_FBox(X1,Y1,X2,Y2:Integer;OutCol,InCol:Byte);
Var
	T:Integer;
Begin
	If Y2<Y1 Then
   Begin
   	T:=Y1;
      Y1:=Y2;
      Y2:=T;
   End;
   For T:=Y1 To Y2 Do
   	P_Line(X1,T,X2,T,InCol);
	P_Box(X1,Y1,X2,Y2,OutCol);
End;

Procedure P_SetRGB(Col,R,G,B:Byte);
Begin
   Port[$3C8]:=Col;
   Port[$3C9]:=R;
   Port[$3C9]:=G;
   Port[$3C9]:=B;
End;

Procedure P_GetRGB(Col:Byte;Var R,G,B:Byte);
Begin
	Port[$3C7]:=Col;
   R:=Port[$3C9];
   G:=Port[$3C9];
   B:=Port[$3C9];
End;

Procedure P_SetPalette(Pal:TPalette);
Var
	C:Byte;
   S,O:Word;
Begin
   S:=Seg(Pal);
   O:=Ofs(Pal);
   P_VSync;
	Asm
   	push DS
      mov DS,S
      mov SI,O
      xor AL,AL
      mov CX,768
      mov DX,03C8h
      out DX,AL
      inc DX
      rep outsb
      pop DS
   End;
End;

Procedure P_GetPalette(Var Pal:TPalette);
Var
	C:Byte;
Begin
   For C:=0 To 255 Do
   Begin
		Port[$3C7]:=C;
	   Pal[C].R:=Port[$3C9];
	   Pal[C].G:=Port[$3C9];
	   Pal[C].B:=Port[$3C9];
	End;
End;

Procedure P_Black(Var Pal:TPalette);
Var
	C:Byte;
Begin
	For C:=0 To 255 Do
   Begin
   	Pal[C].R:=0;
   	Pal[C].G:=0;
   	Pal[C].B:=0;
   End;
End;

Procedure P_FadeOut(Var Pal:TPalette;Chan:Byte);
Var
	C:Byte;
Begin
	For C:=0 To 255 Do
   Begin
   	If (Chan And 1=1) And (Pal[C].R>0) Then
      	Dec(Pal[C].R);
   	If (Chan And 2=2) And (Pal[C].G>0) Then
      	Dec(Pal[C].G);
   	If (Chan And 4=4) And (Pal[C].B>0) Then
      	Dec(Pal[C].B);
   End;
End;

Procedure P_FadeIn(Var Pal:TPalette;DstPal:TPalette;Chan:Byte);
Var
	C:Byte;
Begin
	For C:=0 To 255 Do
   Begin
   	If (Chan And 1=1) And (Pal[C].R<DstPal[C].R) Then
      	Inc(Pal[C].R);
   	If (Chan And 2=2) And (Pal[C].G<DstPal[C].G) Then
      	Inc(Pal[C].G);
   	If (Chan And 4=4) And (Pal[C].B<DstPal[C].B) Then
      	Inc(Pal[C].B);
   End;
End;

Procedure P_Rotate(Var Pal:TPalette;From,Too:Byte;Reverse:Boolean);
Var
	C:Byte;
   Pl:TPalEntry;
Begin
	If Reverse Then
	Begin
   	Pl:=Pal[From];
      For C:=From To Too-1 Do
      	Pal[C]:=Pal[C+1];
      Pal[Too]:=Pl;
	End
	Else
	Begin
		Pl:=Pal[Too];
      For C:=Too DownTo From+1 Do
			Pal[C]:=Pal[C-1];
      Pal[From]:=Pl;
   End;
End;

Procedure P_SavePal(Pal:TPalette;Fil:String);
Var
	F:File;
Begin
	{$I-}
   Assign(F,Fil);
   Rewrite(F,1);
   If IOResult>0 Then
   	Exit;
   BlockWrite(F,Pal,SizeOf(Pal));
   Close(F);
   If IOResult>0 Then
   	Exit;
   {$I+}
End;

Procedure P_LoadPal(Fil:String;Var Pal:TPalette);
Var
	F:File;
Begin
	{$I-}
   Assign(F,Fil);
   Reset(F,1);
   If IOResult>0 Then
   	Exit;
   BlockRead(F,Pal,SizeOf(Pal));
   Close(F);
   If IOResult>0 Then
   	Exit;
   {$I+}
End;

Procedure P_ResetPalette(Var Pal:TPalette);
Begin
	Pal:=TruePal;
End;

End.