{   Pix Graphical User Interface Unit
  

  Programmer                    : Gerard Krijgsman

  Version                       : 1.00
  Date                          : January 15, 1997

  Compiler                      : Borland Pascal 7.0                      }

unit PixGui;

interface

type
  TColors                       = record
                                    Text,
                                    Shadow,
                                    Bright,
                                    Normal,
                                    Dark,
                                    Black,
                                    White       : byte;
                                  end;
  TObject                       = object
                                    X,Y,
                                    Width,
                                    Height      : word;
                                    TextColor,
                                    ShadowColor : byte;
                                    UseShadow   : boolean;
                                    Caption     : string;
                                    constructor Init(PosX,PosY,W,H:word;Cap:string);
                                  end;
  TButton                       = object(TObject)
                                    Touched     : boolean;
                                    Outline     : boolean;
                                    constructor Init(PosX,PosY,W:word;Cap:string;Out:boolean);
                                    procedure Show; virtual;
                                    procedure Touch; virtual;
                                    procedure ShowContents; virtual;
                                    procedure ShowOutline; virtual;
                                    procedure Create; virtual;
                                    function Check:boolean; virtual;
                                  end;
  TCheckBox                     = object(TObject)
                                    Status      : boolean;
                                    Touched     : boolean;
                                    Timer       : longint;
                                    constructor Init(PosX,PosY:word;Cap:string;Stat:boolean);
                                    procedure Show; virtual;
                                    procedure Touch; virtual;
                                    procedure ShowContents; virtual;
                                    procedure ShowOutline; virtual;
                                    procedure Create; virtual;
                                    function Check:boolean; virtual;
                                  end;

var
  Colors                        : TColors;
  TextHeight                    : byte;
  GuiChanged                    : boolean;

procedure P_Panel(X,Y,X2,Y2:word;BackColor:byte;Invert:boolean);
procedure P_Label(X,Y,X2:word;BackColor:byte;Caption:string;TC,SC:byte;Sh:boolean);
procedure P_SetGuiPalette;
function P_GuiChanged:boolean;

implementation
  uses Pix322,PixText,PixMouse;

var
  ClockTicks                    : longint absolute $40:$6C;

procedure P_Panel(X,Y,X2,Y2:word;BackColor:byte;Invert:boolean);
begin
  P_Fbox(X,Y,X2,Y2,Colors.Normal,BackColor);
  if not Invert then
    begin
      P_Hline(X,X2-1,Y,Colors.Bright);
      P_Vline(Y,Y2-1,X,Colors.Bright);
      P_Hline(X+1,X2,Y2,Colors.Dark);
      P_Vline(Y+1,Y2,X2,Colors.Dark);
    end
  else
    begin
      P_Hline(X,X2-1,Y,Colors.Dark);
      P_Vline(Y,Y2-1,X,Colors.Dark);
      P_Hline(X+1,X2,Y2,Colors.Bright);
      P_Vline(Y+1,Y2,X2,Colors.Bright);
    end;
end;

procedure P_Label(X,Y,X2:word;BackColor:byte;Caption:string;TC,SC:byte;Sh:boolean);
begin
  P_Panel(X,Y,X2,Y+TextHeight+5,BackColor,TRUE);
  if Sh then
    P_Writes(X+(((X2-X)-P_Length(Caption)) div 2),Y+2,TC,SC,Caption)
  else
    P_Write(X+(((X2-X)-P_Length(Caption)) div 2),Y+2,TC,Caption);
end;

procedure PixPalette; external; {$L PIXPAL.OBJ}

procedure P_SetGuiPalette;
var
  GuiPalette    : ^TPalette;
begin
  GuiPalette := @PixPalette;
  P_SetPalette(GuiPalette^);
end;

function P_GuiChanged:boolean;
begin
  P_GuiChanged := GuiChanged;
  GuiChanged := FALSE;
end;

constructor TObject.Init;
begin
  X := PosX;
  Y := PosY;
  Width := W;
  Height := H;
  Caption := Cap;
  TextColor := Colors.Text;
  ShadowColor := Colors.Shadow;
  UseShadow := TRUE;
end;

constructor TButton.Init;
begin
  inherited Init(PosX,PosY,W,TextHeight+5,Cap);
  Touched := FALSE;
  Outline := Out;
end;

procedure TButton.Show;
begin
  P_Panel(X,Y,X+Width-1,Y+Height-1,Colors.Normal,FALSE);
  GuiChanged := TRUE;
end;

procedure TButton.Touch;
begin
  P_Panel(X,Y,X+Width-1,Y+Height-1,Colors.Normal,TRUE);
  GuiChanged := TRUE;
end;

procedure TButton.ShowContents;
begin
  if UseShadow then
    P_Writes(X+(((Width-1)-P_Length(Caption)) div 2),Y+(((Height-1)-TextHeight) div 2),TextColor,ShadowColor,Caption)
  else
    P_Write(X+(((Width-1)-P_Length(Caption)) div 2),Y+(((Height-1)-TextHeight) div 2),TextColor,Caption);
  GuiChanged := TRUE;
end;

procedure TButton.ShowOutline;
begin
  P_Panel(X-2,Y-2,X+Width+1,Y+Height+1,Colors.Black,TRUE);
  GuiChanged := TRUE;
end;

procedure TButton.Create;
begin
  if Outline then ShowOutline;
  Show;
  ShowContents;
end;

function TButton.Check:boolean;
begin
  Check := FALSE;
  if not Touched then
    begin
      if (MouseX >= X) and (MouseX <= (X+Width-1)) and (MouseY >= Y) and
         (MouseY <= (Y+Height-1)) and MouseB1 then
        begin
          Touched := TRUE;
          Touch;
          ShowContents;
        end;
    end
  else
    if (MouseX >= X) and (MouseX <= (X+Width-1)) and (MouseY >= Y) and
       (MouseY <= (Y+Height-1)) and not MouseB1 then
      begin
        Check := TRUE;
        Touched := FALSE;
        Show;
        ShowContents;
      end
    else
      if (MouseX < X) or (MouseX > (X+Width-1)) or (MouseY < Y) or
         (MouseY > (Y+Height-1)) then
        begin
          Touched := FALSE;
          Show;
          ShowContents;
        end;
end;

constructor TCheckBox.Init;
begin
  inherited Init(PosX,PosY,10,10,Cap);
  Status := Stat;
  Touched := FALSE;
  Timer := 0;
end;

procedure TCheckBox.Show;
begin
  P_Panel(X+2,Y+2,X+Width-3,Y+Height-3,Colors.Normal,FALSE);
  GuiChanged := TRUE;
end;

procedure TCheckBox.Touch;
begin
  P_Panel(X+2,Y+2,X+Width-3,Y+Height-3,Colors.Normal,TRUE);
  GuiChanged := TRUE;
end;

procedure TCheckBox.ShowContents;
begin
  if UseShadow then
    P_Writes(X+Width+4,Y+(((Height-1)-TextHeight) div 2),TextColor,ShadowColor,Caption)
  else
    P_Write(X+Width+4,Y+(((Height-1)-TextHeight) div 2),TextColor,Caption);
  GuiChanged := TRUE;
end;

procedure TCheckBox.ShowOutline;
begin
  P_Panel(X,Y,X+Width-1,Y+Height-1,Colors.Black,TRUE);
  GuiChanged := TRUE;
end;

procedure TCheckBox.Create;
begin
  ShowOutline;
  if Status then Touch else Show;
  ShowContents;
end;

function TCheckBox.Check:boolean;
begin
  Check := FALSE;
  if (MouseX >= X+2) and (MouseX <= (X+Width-3)) and (MouseY >= Y+2) and
     (MouseY <= (Y+Height-3)) and MouseB1 and (Timer+2 < ClockTicks) then
    begin
      if not Touched and not Status then Touch;
      Touched := TRUE;
    end;
  if ((MouseX <= X) or (MouseX >= (X+Width-1)) or (MouseY <= Y) or
      (MouseY >= (Y+Height-1))) and Touched then
    begin
      if not Status then Show;
      Touched := FALSE;
    end;
  if (MouseX >= X+2) and (MouseX <= (X+Width-3)) and (MouseY >= Y+2) and
     (MouseY <= (Y+Height-3)) and not MouseB1 and not Status and Touched then
    begin
      Timer := ClockTicks;
      Status := TRUE;
      Check := TRUE;
      Touched := FALSE;
    end;
  if (MouseX >= X+2) and (MouseX <= (X+Width-3)) and (MouseY >= Y+2) and
     (MouseY <= (Y+Height-3)) and not MouseB1 and Status and Touched then
    begin
      Timer := ClockTicks;
      Show;
      Status := FALSE;
      Check := TRUE;
      Touched := FALSE;
    end;
{  if (MouseX >= X) and (MouseX <= (X+Width-1)) and (MouseY >= Y) and
     (MouseY <= (Y+Height-1)) and not MouseB1 and Touched then
    begin
      Status := TRUE;
      Check := TRUE;
    end;
  if (MouseX >= X) and (MouseX <= (X+Width-1)) and (MouseY >= Y) and
     (MouseY <= (Y+Height-1)) and not MouseB1 and not Touched then
    begin
      Show;
      Status := TRUE;
      Check := TRUE;
    end;}
end;

begin
  with Colors do
    begin
      Text := 31;
      Shadow := 16;
      Bright := 64;
      Normal := 65;
      Dark := 66;
      Black := 16;
      White := 31;
    end;
  TextHeight := 5;
  GuiChanged := FALSE;
end.